package com.emato.cuspay.util;

import com.emato.cuspay.support.msg.resp.ResponseMessage;
import com.emato.cuspay.support.msg.resp.ResponseStatus;
import org.apache.commons.lang3.StringUtils;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

/**
 * @author zx
 * @version 1.0
 * 2018-05-15 16:48
 */
public abstract class Validator {
    /**
     * 非空校验
     *
     * @param beVerified 要验证的字段 <code>{"字段名","错误消息"}</code>
     * @param valideDate       验证数据
     * @return
     */
    public static ResponseMessage isEmpty(Map<String, Object> beVerified, Map<String, Object> valideDate) {


        /**
         * 验证数据为空
         */
        if (valideDate == null) {
            return new ResponseMessage.Builder()
                        .setCode(ResponseStatus.ERROR.getItem())
                        .setMsg("验证数据为空")
                        .build();
        }

        /**
         * 没有要验证的
         */
        if (beVerified == null) {
            return new ResponseMessage.Builder()
                    .setCode(ResponseStatus.ERROR.getItem())
                    .setMsg("验证数据为空")
                    .build();
        }

        for (String key : beVerified.keySet()) {
            //验证数据没有要验证的字段, 或有要验证的字段,但字段为空
            if (!valideDate.containsKey(key) ||
                    (valideDate.containsKey(key) && StringUtils.isBlank(String.valueOf(valideDate.get(key)))) ||
                    String.valueOf(valideDate.get(key)).equals("null")) {
                return  ResponseMessage.builder(ResponseStatus.ERROR.getItem(), beVerified.get(key)+"为空").build();
            }
        }
        return new ResponseMessage.Builder()
                    .setCode(ResponseStatus.SUCCESS.getItem())
                    .build();
    }

    /**
     * 验证身份证号
     * 大陆是18位
     * 香港是15位
     */
    public static boolean isCardNo(String str)throws PatternSyntaxException {
        String regExp = "^[1-9]\\d{5}(18|19|([23]\\d))\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$|(^[1-9]\\d{5}\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{2}$)";
        Pattern p = Pattern.compile(regExp);
        Matcher m = p.matcher(str);
        return m.matches();
    }
}
