package com.emato.cuspay.wx.service.impl;


import com.emato.cuspay.common.contant.TablePrimaryKeyPrefix;
import com.emato.cuspay.common.core.db.IdWorkerAide;
import com.emato.cuspay.dao.mapper.merch.MerchPayCfgMapper;
import com.emato.cuspay.dao.mapper.wx.WxCbPayDocMapper;
import com.emato.cuspay.entity.merch.MerchPayCfg;
import com.emato.cuspay.wx.entity.WxCbPayDoc;
import com.emato.cuspay.support.msg.resp.ResponseMessage;
import com.emato.cuspay.support.msg.resp.ResponseStatus;
import com.emato.cuspay.util.jackson.JacksonUtil;
import com.emato.cuspay.wx.common.WxDict;
import com.emato.cuspay.wx.service.WxCbPayDocService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;

/**
 * @author huangyq
 * @version 1.0
 * 2018-05-18 16:05
 */
@Service
public class WxCbPayDocServiceImpl implements WxCbPayDocService {
    private static final Logger logger = LoggerFactory.getLogger(WxCbPayDocServiceImpl.class);

    @Autowired
    private WxCbPayDocMapper wxCbPayDocMapper;

    @Autowired
    private MerchPayCfgMapper merchPayCfgMapper;

    /**
     * 支付单信息入库
     * @param wxCbPayDoc 微信支付单证
     * @return
     */
    public ResponseMessage addWxCbPayDoc(WxCbPayDoc wxCbPayDoc) {
        wxCbPayDoc.setWxPaySn(TablePrimaryKeyPrefix.wx_cb_pay_type + IdWorkerAide.nextId());
        //设置支付单状态 待审核
        wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_00.getItem());
        String merchSn = wxCbPayDoc.getMerchSn();
        try {
            List<MerchPayCfg> merchPayCfgList = merchPayCfgMapper.getMerchPayCfgByMerchsn(merchSn);
            if(merchPayCfgList.size() == 0){
                return  new ResponseMessage.Builder().setCode(ResponseStatus.ERROR.getItem()).setMsg("商户编号为"+merchSn+"的支付配置信息不存在")
                        .build();
            }
        } catch (Exception e) {
            logger.error("查询商户支付配置信息异常", e);
            return  new ResponseMessage.Builder().setCode(ResponseStatus.ERROR.getItem()).setMsg("查询商户支付配置信息异常")
                    .build();
        }

        try {
            int result = wxCbPayDocMapper.insertSelective(wxCbPayDoc);
            if (result > 0) {
                HashMap<String,String> resultMap = new HashMap<>();
                resultMap.put("docStatus", WxDict.PaymentDocStatus.i_00.getItemName());

                return new ResponseMessage.Builder().setCode(ResponseStatus.SUCCESS.getItem()).setMsg("入库成功")
                        .setData(JacksonUtil.toJson(resultMap)).build();
            }
        } catch (Exception e) {
            logger.error("新增支付单入库信息异常", e);
            return  new ResponseMessage.Builder().setCode(ResponseStatus.ERROR.getItem()).setMsg("新增支付单入库信息异常")
                    .build();
        }
        return  new ResponseMessage.Builder().setCode(ResponseStatus.ERROR.getItem()).setMsg("入库失败")
                .build();
    }

    public List<WxCbPayDoc> selectBeDeclaredData(Integer limit) {
        return wxCbPayDocMapper.selectBeDeclaredData(limit);
    }

    public int updateWxCbPay(WxCbPayDoc wxCbPayDoc) {
        try {
            return wxCbPayDocMapper.updateByPrimaryKeySelective(wxCbPayDoc);
        } catch (Exception e) {
            logger.error("更新支付单数据失败", e);
            throw new RuntimeException("更新支付单数据失败");
        }
    }

    public int updateBatch(List<WxCbPayDoc> wxCbPayDocs) {
        return  wxCbPayDocMapper.updateBatch(wxCbPayDocs);
    }

    public List<WxCbPayDoc> selectPayDocByDocStatus(Integer limit) {
        return wxCbPayDocMapper.selectPayDocByDocStatus(limit);
    }
}
